// Copyright (c) 2022 by István Váradi

// This file is part of VSCPL, a simple cross-platform utility library

// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:

// 1. Redistributions of source code must retain the above copyright notice, this
//    list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright notice,
//    this list of conditions and the following disclaimer in the documentation
//    and/or other materials provided with the distribution.

// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
// DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

// The views and conclusions contained in the software and documentation are those
// of the authors and should not be interpreted as representing official policies,
// either expressed or implied, of the FreeBSD Project.

#pragma once

//------------------------------------------------------------------------------

#include <windows.h>

//------------------------------------------------------------------------------

namespace hu { namespace varadiistvan { namespace scpl { namespace io {

//------------------------------------------------------------------------------

/**
 * An object to access the WSA functions through it. It is a singleton, and its
 * constructor initializes the WSA subsystem, while its destructor cleans it
 * up.
 */
class WSAInterface
{
public:
    /**
     * Get the only instance of this class
     */
    static WSAInterface& get();

private:
    /**
     * Construct the interface.
     */
    WSAInterface();

    /**
     * Destroy the interface.
     */
    ~WSAInterface();

public:
    /**
     * Get the invalid socket handle as a HANDLE.
     */
    HANDLE getInvalidSocket();

    /**
     * Determine if the given handle represents an invalid socket.
     */
    bool isInvalidSocket(HANDLE handle);

    /**
     * Create a socket.
     */
    HANDLE socket(int af, int type, int protocol);

    /**
     * Set the given socket into non-blocking mode and make the given event to
     * be triggered by the events in the given event mask
     */
    int eventSelect(HANDLE socket, HANDLE event, long eventMask);

    /**
     * Set a socket option
     */
    int setsockopt(HANDLE socket, int level, int optname,
                   const char* optval, int optlen);

    /**
     * Bind the given socket handle to the given address.
     */
    int bind(HANDLE socket, const struct sockaddr* name, int namelen);

    /**
     * Start listening on the given socket.
     */
    int listen(HANDLE socket, int backlog = 5);

    /**
     * Accept a socket on the given server socket.
     */
    HANDLE accept(HANDLE socket,
                  struct sockaddr* addr = nullptr, int* addrlen = nullptr);

    /**
     * Try to connect to the given address using the given socket.
     */
    int connect(HANDLE socket, const struct sockaddr* name, int namelen);

    /**
     * Close the givern socket.
     */
    int closesocket(HANDLE socket);
};

//------------------------------------------------------------------------------
// Inline definitions
//------------------------------------------------------------------------------

inline WSAInterface& WSAInterface::get()
{
    static WSAInterface wsa;
    return wsa;
}

//------------------------------------------------------------------------------

} /* namespace hu::varadiistvan::scpl::io */ } /* namespace hu::varadiistvan::scpl */ } /* namespace hu::varadiistvan */ } /* namespace hu */

//------------------------------------------------------------------------------

// Local Variables:
// mode: C++
// c-basic-offset: 4
// indent-tabs-mode: nil
// End:
